@@ -119,7 +119,7 @@ void Archive::BrokenHeaderMsg()
 }
 
 
-void Archive::UnkEncVerMsg(const wchar *Name,const wchar *Info)
+void Archive::UnkEncVerMsg(const std::wstring &Name,const std::wstring &Info)
 {
   uiMsg(UIERROR_UNKNOWNENCMETHOD,FileName,Name,Info);
   ErrHandler.SetErrorCode(RARX_WARNING);
@@ -222,7 +222,7 @@ size_t Archive::ReadHeader15()
   {
     case HEAD_MAIN:
       MainHead.Reset();
-      *(BaseBlock *)&MainHead=ShortBlock;
+      MainHead.SetBaseBlock(ShortBlock);
       MainHead.HighPosAV=Raw.Get2();
       MainHead.PosAV=Raw.Get4();
 
@@ -248,13 +248,17 @@ size_t Archive::ReadHeader15()
         FileHeader *hd=FileBlock ? &FileHead:&SubHead;
         hd->Reset();
 
-        *(BaseBlock *)hd=ShortBlock;
+        hd->SetBaseBlock(ShortBlock);
 
         hd->SplitBefore=(hd->Flags & LHD_SPLIT_BEFORE)!=0;
         hd->SplitAfter=(hd->Flags & LHD_SPLIT_AFTER)!=0;
         hd->Encrypted=(hd->Flags & LHD_PASSWORD)!=0;
         hd->SaltSet=(hd->Flags & LHD_SALT)!=0;
+        
+        // RAR versions earlier than 2.0 do not set the solid flag
+        // in file header. They use only a global solid archive flag.
         hd->Solid=FileBlock && (hd->Flags & LHD_SOLID)!=0;
+
         hd->SubBlock=!FileBlock && (hd->Flags & LHD_SOLID)!=0;
         hd->Dir=(hd->Flags & LHD_WINDOWMASK)==LHD_DIRECTORY;
         hd->WinSize=hd->Dir ? 0:0x10000<<((hd->Flags & LHD_WINDOWMASK)>>5);
@@ -303,7 +307,7 @@ size_t Archive::ReadHeader15()
         if (hd->HostOS==HOST_UNIX && (hd->FileAttr & 0xF000)==0xA000)
         {
           hd->RedirType=FSREDIR_UNIXSYMLINK;
-          *hd->RedirName=0;
+          hd->RedirName.clear();
         }
 
         hd->Inherited=!FileBlock && (hd->SubFlags & SUBHEAD_FLAGS_INHERITED)!=0;
@@ -330,27 +334,26 @@ size_t Archive::ReadHeader15()
         if (hd->UnknownUnpSize)
           hd->UnpSize=INT64NDF;
 
-        char FileName[NM*4];
-        size_t ReadNameSize=Min(NameSize,ASIZE(FileName)-1);
-        Raw.GetB((byte *)FileName,ReadNameSize);
-        FileName[ReadNameSize]=0;
+        size_t ReadNameSize=Min(NameSize,MAXPATHSIZE);
+        std::string FileName(ReadNameSize,0);
+        Raw.GetB((byte *)&FileName[0],ReadNameSize);
 
         if (FileBlock)
         {
-          *hd->FileName=0;
+          hd->FileName.clear();
           if ((hd->Flags & LHD_UNICODE)!=0)
           {
             EncodeFileName NameCoder;
-            size_t Length=strlen(FileName);
+            size_t Length=strlen(FileName.data());
             Length++;
             if (ReadNameSize>Length)
-              NameCoder.Decode(FileName,ReadNameSize,(byte *)FileName+Length,
-                               ReadNameSize-Length,hd->FileName,
-                               ASIZE(hd->FileName));
+              NameCoder.Decode(FileName.data(),ReadNameSize,
+                               (byte *)&FileName[Length],
+                               ReadNameSize-Length,hd->FileName);
           }
 
-          if (*hd->FileName==0)
-            ArcCharToWide(FileName,hd->FileName,ASIZE(hd->FileName),ACTW_OEM);
+          if (hd->FileName.empty())
+            ArcCharToWide(FileName.data(),hd->FileName,ACTW_OEM);
 
 #ifndef SFX_MODULE
           ConvertNameCase(hd->FileName);
@@ -359,7 +362,7 @@ size_t Archive::ReadHeader15()
         }
         else
         {
-          CharToWide(FileName,hd->FileName,ASIZE(hd->FileName));
+          CharToWide(FileName.data(),hd->FileName);
 
           // Calculate the size of optional data.
           int DataSize=int(hd->HeadSize-NameSize-SIZEOF_FILEHEAD3);
@@ -370,14 +373,15 @@ size_t Archive::ReadHeader15()
           {
             // Here we read optional additional fields for subheaders.
             // They are stored after the file name and before salt.
-            hd->SubData.Alloc(DataSize);
-            Raw.GetB(&hd->SubData[0],DataSize);
+            hd->SubData.resize(DataSize);
+            Raw.GetB(hd->SubData.data(),DataSize);
 
           }
 
           if (hd->CmpName(SUBHEAD_TYPE_CMT))
             MainComment=true;
         }
+
         if ((hd->Flags & LHD_SALT)!=0)
           Raw.GetB(hd->Salt,SIZE_SALT30);
         hd->mtime.SetDos(FileTime);
@@ -420,7 +424,7 @@ size_t Archive::ReadHeader15()
         NextBlockPos=SafeAdd(NextBlockPos,hd->PackSize,0);
 
         bool CRCProcessedOnly=hd->CommentInHeader;
-        ushort HeaderCRC=Raw.GetCRC15(CRCProcessedOnly);
+        uint HeaderCRC=Raw.GetCRC15(CRCProcessedOnly);
         if (hd->HeadCRC!=HeaderCRC)
         {
           BrokenHeader=true;
@@ -437,7 +441,7 @@ size_t Archive::ReadHeader15()
       }
       break;
     case HEAD_ENDARC:
-      *(BaseBlock *)&EndArcHead=ShortBlock;
+      EndArcHead.SetBaseBlock(ShortBlock);
       EndArcHead.NextVolume=(EndArcHead.Flags & EARC_NEXT_VOLUME)!=0;
       EndArcHead.DataCRC=(EndArcHead.Flags & EARC_DATACRC)!=0;
       EndArcHead.RevSpace=(EndArcHead.Flags & EARC_REVSPACE)!=0;
@@ -449,14 +453,14 @@ size_t Archive::ReadHeader15()
       break;
 #ifndef SFX_MODULE
     case HEAD3_CMT:
-      *(BaseBlock *)&CommHead=ShortBlock;
+      CommHead.SetBaseBlock(ShortBlock);
       CommHead.UnpSize=Raw.Get2();
       CommHead.UnpVer=Raw.Get1();
       CommHead.Method=Raw.Get1();
       CommHead.CommCRC=Raw.Get2();
       break;
     case HEAD3_PROTECT:
-      *(BaseBlock *)&ProtectHead=ShortBlock;
+      ProtectHead.SetBaseBlock(ShortBlock);
       ProtectHead.DataSize=Raw.Get4();
       ProtectHead.Version=Raw.Get1();
       ProtectHead.RecSectors=Raw.Get2();
@@ -465,26 +469,13 @@ size_t Archive::ReadHeader15()
       NextBlockPos+=ProtectHead.DataSize;
       break;
     case HEAD3_OLDSERVICE: // RAR 2.9 and earlier.
-      *(BaseBlock *)&SubBlockHead=ShortBlock;
+      SubBlockHead.SetBaseBlock(ShortBlock);
       SubBlockHead.DataSize=Raw.Get4();
       NextBlockPos+=SubBlockHead.DataSize;
       SubBlockHead.SubType=Raw.Get2();
       SubBlockHead.Level=Raw.Get1();
       switch(SubBlockHead.SubType)
       {
-        case UO_HEAD:
-          *(SubBlockHeader *)&UOHead=SubBlockHead;
-          UOHead.OwnerNameSize=Raw.Get2();
-          UOHead.GroupNameSize=Raw.Get2();
-          if (UOHead.OwnerNameSize>=ASIZE(UOHead.OwnerName))
-            UOHead.OwnerNameSize=ASIZE(UOHead.OwnerName)-1;
-          if (UOHead.GroupNameSize>=ASIZE(UOHead.GroupName))
-            UOHead.GroupNameSize=ASIZE(UOHead.GroupName)-1;
-          Raw.GetB(UOHead.OwnerName,UOHead.OwnerNameSize);
-          Raw.GetB(UOHead.GroupName,UOHead.GroupNameSize);
-          UOHead.OwnerName[UOHead.OwnerNameSize]=0;
-          UOHead.GroupName[UOHead.GroupNameSize]=0;
-          break;
         case NTACL_HEAD:
           *(SubBlockHeader *)&EAHead=SubBlockHead;
           EAHead.UnpSize=Raw.Get4();
@@ -499,10 +490,13 @@ size_t Archive::ReadHeader15()
           StreamHead.Method=Raw.Get1();
           StreamHead.StreamCRC=Raw.Get4();
           StreamHead.StreamNameSize=Raw.Get2();
-          if (StreamHead.StreamNameSize>=ASIZE(StreamHead.StreamName))
-            StreamHead.StreamNameSize=ASIZE(StreamHead.StreamName)-1;
-          Raw.GetB(StreamHead.StreamName,StreamHead.StreamNameSize);
-          StreamHead.StreamName[StreamHead.StreamNameSize]=0;
+
+          const size_t MaxStreamName20=260; // Maximum allowed stream name in RAR 2.x format.
+          if (StreamHead.StreamNameSize>MaxStreamName20)
+            StreamHead.StreamNameSize=MaxStreamName20;
+
+          StreamHead.StreamName.resize(StreamHead.StreamNameSize);
+          Raw.GetB(&StreamHead.StreamName[0],StreamHead.StreamNameSize);
           break;
       }
       break;
@@ -513,11 +507,15 @@ size_t Archive::ReadHeader15()
       break;
   }
 
-  ushort HeaderCRC=Raw.GetCRC15(false);
+  uint HeaderCRC=Raw.GetCRC15(false);
 
   // Old AV header does not have header CRC properly set.
+  // Old Unix owners header didn't include string fields into header size,
+  // but included them into CRC, so it couldn't be verified with generic
+  // approach here.
   if (ShortBlock.HeadCRC!=HeaderCRC && ShortBlock.HeaderType!=HEAD3_SIGN &&
-      ShortBlock.HeaderType!=HEAD3_AV)
+      ShortBlock.HeaderType!=HEAD3_AV && 
+      (ShortBlock.HeaderType!=HEAD3_OLDSERVICE || SubBlockHead.SubType!=UO_HEAD))
   {
     bool Recovered=false;
     if (ShortBlock.HeaderType==HEAD_ENDARC && EndArcHead.RevSpace)
@@ -651,7 +649,7 @@ size_t Archive::ReadHeader50()
   }
 
   int SizeToRead=int(BlockSize);
-  SizeToRead-=FirstReadSize-SizeBytes-4; // Adjust overread size bytes if any.
+  SizeToRead-=int(FirstReadSize-SizeBytes-4); // Adjust overread size bytes if any.
   uint HeaderSize=4+SizeBytes+(uint)BlockSize;
 
   if (SizeToRead<0 || HeaderSize<SIZEOF_SHORTBLOCKHEAD5)
@@ -716,13 +714,12 @@ size_t Archive::ReadHeader50()
   {
     case HEAD_CRYPT:
       {
-        *(BaseBlock *)&CryptHead=ShortBlock;
+        CryptHead.SetBaseBlock(ShortBlock);
         uint CryptVersion=(uint)Raw.GetV();
         if (CryptVersion>CRYPT_VERSION)
         {
-          wchar Info[20];
-          swprintf(Info,ASIZE(Info),L"h%u",CryptVersion);
-          UnkEncVerMsg(FileName,Info);
+          UnkEncVerMsg(FileName,L"h" + std::to_wstring(CryptVersion));
+          FailedHeaderDecryption=true;
           return 0;
         }
         uint EncFlags=(uint)Raw.GetV();
@@ -730,9 +727,8 @@ size_t Archive::ReadHeader50()
         CryptHead.Lg2Count=Raw.Get1();
         if (CryptHead.Lg2Count>CRYPT5_KDF_LG2_COUNT_MAX)
         {
-          wchar Info[20];
-          swprintf(Info,ASIZE(Info),L"hc%u",CryptHead.Lg2Count);
-          UnkEncVerMsg(FileName,Info);
+          UnkEncVerMsg(FileName,L"hc" + std::to_wstring(CryptHead.Lg2Count));
+          FailedHeaderDecryption=true;
           return 0;
         }
 
@@ -744,12 +740,8 @@ size_t Archive::ReadHeader50()
           byte csum[SIZE_PSWCHECK_CSUM];
           Raw.GetB(csum,SIZE_PSWCHECK_CSUM);
 
-          sha256_context ctx;
-          sha256_init(&ctx);
-          sha256_process(&ctx, CryptHead.PswCheck, SIZE_PSWCHECK);
-
           byte Digest[SHA256_DIGEST_SIZE];
-          sha256_done(&ctx, Digest);
+          sha256_get(CryptHead.PswCheck, SIZE_PSWCHECK, Digest);
 
           CryptHead.UsePswCheck=memcmp(csum,Digest,SIZE_PSWCHECK_CSUM)==0;
         }
@@ -759,7 +751,7 @@ size_t Archive::ReadHeader50()
     case HEAD_MAIN:
       {
         MainHead.Reset();
-        *(BaseBlock *)&MainHead=ShortBlock;
+        MainHead.SetBaseBlock(ShortBlock);
         uint ArcFlags=(uint)Raw.GetV();
 
         Volume=(ArcFlags & MHFL_VOLUME)!=0;
@@ -837,9 +829,14 @@ size_t Archive::ReadHeader50()
         // we may need to use the compression algorithm 15 in the future,
         // but it was already used in RAR 1.5 and Unpack needs to distinguish
         // them.
-        hd->UnpVer=(CompInfo & 0x3f) + 50;
-        if (hd->UnpVer!=50) // Only 5.0 compression is known now.
-          hd->UnpVer=VER_UNKNOWN;
+        uint UnpVer=(CompInfo & 0x3f);
+        if (UnpVer==0)
+          hd->UnpVer=VER_PACK5;
+        else
+          if (UnpVer==1)
+            hd->UnpVer=VER_PACK7;
+          else
+            hd->UnpVer=VER_UNKNOWN;
 
         hd->HostOS=(byte)Raw.GetV();
         size_t NameSize=(size_t)Raw.GetV();
@@ -857,14 +854,29 @@ size_t Archive::ReadHeader50()
         hd->SubBlock=(hd->Flags & HFL_CHILD)!=0;
         hd->Solid=FileBlock && (CompInfo & FCI_SOLID)!=0;
         hd->Dir=(hd->FileFlags & FHFL_DIRECTORY)!=0;
-        hd->WinSize=hd->Dir ? 0:size_t(0x20000)<<((CompInfo>>10)&0xf);
+        if (hd->Dir || UnpVer>1)
+          hd->WinSize=0;
+        else
+        {
+          hd->WinSize=0x20000ULL<<((CompInfo>>10)&(UnpVer==0 ? 0x0f:0x1f));
+          if (UnpVer==1)
+          {
+            hd->WinSize+=hd->WinSize/32*((CompInfo>>15)&0x1f);
 
-        char FileName[NM*4];
-        size_t ReadNameSize=Min(NameSize,ASIZE(FileName)-1);
-        Raw.GetB((byte *)FileName,ReadNameSize);
-        FileName[ReadNameSize]=0;
+            // RAR7 header with RAR5 compression. Needed to append RAR7 files
+            // to RAR5 solid stream if new dictionary is larger than existing.
+            if ((CompInfo & FCI_RAR5_COMPAT)!=0)
+              hd->UnpVer=VER_PACK5;
+            if (hd->WinSize>UNPACK_MAX_DICT)
+              hd->UnpVer=VER_UNKNOWN;
+          }
+        }
 
-        UtfToWide(FileName,hd->FileName,ASIZE(hd->FileName));
+        size_t ReadNameSize=Min(NameSize,MAXPATHSIZE);
+        std::string FileName(ReadNameSize,0);
+        Raw.GetB((byte *)&FileName[0],ReadNameSize);
+
+        UtfToWide(FileName.data(),hd->FileName);
 
         // Should do it before converting names, because extra fields can
         // affect name processing, like in case of NTFS streams.
@@ -889,7 +901,7 @@ size_t Archive::ReadHeader50()
       break;
     case HEAD_ENDARC:
       {
-        *(BaseBlock *)&EndArcHead=ShortBlock;
+        EndArcHead.SetBaseBlock(ShortBlock);
         uint ArcFlags=(uint)Raw.GetV();
         EndArcHead.NextVolume=(ArcFlags & EHFL_NEXTVOLUME)!=0;
         EndArcHead.StoreVolNumber=false;
@@ -921,7 +933,7 @@ void Archive::RequestArcPassword(RarCheckPassword *CheckPwd)
         *PasswordA=0;
         if (Cmd->Callback(UCM_NEEDPASSWORD,Cmd->UserData,(LPARAM)PasswordA,ASIZE(PasswordA))==-1)
           *PasswordA=0;
-        GetWideName(PasswordA,NULL,PasswordW,ASIZE(PasswordW));
+        CharToWide(PasswordA,PasswordW,ASIZE(PasswordW));
         cleandata(PasswordA,sizeof(PasswordA));
       }
       Cmd->Password.Set(PasswordW);
@@ -996,19 +1008,14 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,const BaseBlock *bb)
             if ((Flags & MHEXTRA_METADATA_NAME)!=0)
             {
               uint64 NameSize=Raw->GetV();
-              if (NameSize<0x10000) // Prevent excessive allocation.
+              if (NameSize>0 && NameSize<MAXPATHSIZE) // Prevent excessive allocation.
               {
-                std::vector<char> NameU((size_t)NameSize); // UTF-8 name.
+                std::string NameU((size_t)NameSize,0); // UTF-8 name.
                 Raw->GetB(&NameU[0],(size_t)NameSize);
                 // If starts from 0, the name was longer than reserved space
                 // when saving this extra field.
                 if (NameU[0]!=0)
-                {
-                  NameU.push_back(0);
-                  std::vector<wchar> NameW(NameU.size()*4);
-                  UtfToWide(&NameU[0],&NameW[0],NameW.size());
-                  hd->OrigName.assign(&NameW[0]);
-                }
+                  UtfToWide(&NameU[0],hd->OrigName);
               }
             }
             if ((Flags & MHEXTRA_METADATA_CTIME)!=0)
@@ -1034,11 +1041,7 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,const BaseBlock *bb)
             FileHeader *hd=(FileHeader *)bb;
             uint EncVersion=(uint)Raw->GetV();
             if (EncVersion>CRYPT_VERSION)
-            {
-              wchar Info[20];
-              swprintf(Info,ASIZE(Info),L"x%u",EncVersion);
-              UnkEncVerMsg(hd->FileName,Info);
-            }
+              UnkEncVerMsg(hd->FileName,L"x" + std::to_wstring(EncVersion));
             else
             {
               uint Flags=(uint)Raw->GetV();
@@ -1046,11 +1049,7 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,const BaseBlock *bb)
               hd->UseHashKey=(Flags & FHEXTRA_CRYPT_HASHMAC)!=0;
               hd->Lg2Count=Raw->Get1();
               if (hd->Lg2Count>CRYPT5_KDF_LG2_COUNT_MAX)
-              {
-                wchar Info[20];
-                swprintf(Info,ASIZE(Info),L"xc%u",hd->Lg2Count);
-                UnkEncVerMsg(hd->FileName,Info);
-              }
+                UnkEncVerMsg(hd->FileName,L"xc" + std::to_wstring(hd->Lg2Count));
               Raw->GetB(hd->Salt,SIZE_SALT50);
               Raw->GetB(hd->InitV,SIZE_INITV);
               if (hd->UsePswCheck)
@@ -1067,12 +1066,8 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,const BaseBlock *bb)
                 byte csum[SIZE_PSWCHECK_CSUM];
                 Raw->GetB(csum,SIZE_PSWCHECK_CSUM);
 
-                sha256_context ctx;
-                sha256_init(&ctx);
-                sha256_process(&ctx, hd->PswCheck, SIZE_PSWCHECK);
-
                 byte Digest[SHA256_DIGEST_SIZE];
-                sha256_done(&ctx, Digest);
+                sha256_get(hd->PswCheck, SIZE_PSWCHECK, Digest);
 
                 hd->UsePswCheck=memcmp(csum,Digest,SIZE_PSWCHECK_CSUM)==0;
 
@@ -1138,31 +1133,27 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,const BaseBlock *bb)
             if (Version!=0)
             {
               hd->Version=true;
-
-              wchar VerText[20];
-              swprintf(VerText,ASIZE(VerText),L";%u",Version);
-              wcsncatz(hd->FileName,VerText,ASIZE(hd->FileName));
+              hd->FileName += L';' + std::to_wstring(Version);
             }
           }
           break;
         case FHEXTRA_REDIR:
           {
-            hd->RedirType=(FILE_SYSTEM_REDIRECT)Raw->GetV();
+            FILE_SYSTEM_REDIRECT RedirType=(FILE_SYSTEM_REDIRECT)Raw->GetV();
             uint Flags=(uint)Raw->GetV();
-            hd->DirTarget=(Flags & FHEXTRA_REDIR_DIR)!=0;
             size_t NameSize=(size_t)Raw->GetV();
 
-            char UtfName[NM*4];
-            *UtfName=0;
-            if (NameSize<ASIZE(UtfName)-1)
+            if (NameSize>0 && NameSize<MAXPATHSIZE)
             {
-              Raw->GetB(UtfName,NameSize);
-              UtfName[NameSize]=0;
-            }
+              std::string UtfName(NameSize,0);
+              hd->RedirType=RedirType;
+              hd->DirTarget=(Flags & FHEXTRA_REDIR_DIR)!=0;
+              Raw->GetB(&UtfName[0],NameSize);
+              UtfToWide(&UtfName[0],hd->RedirName);
 #ifdef _WIN_ALL
-            UnixSlashToDos(UtfName,UtfName,ASIZE(UtfName));
+              UnixSlashToDos(hd->RedirName,hd->RedirName);
 #endif
-            UtfToWide(UtfName,hd->RedirName,ASIZE(hd->RedirName));
+            }
           }
           break;
         case FHEXTRA_UOWNER:
@@ -1219,8 +1210,8 @@ void Archive::ProcessExtra50(RawRead *Raw,size_t ExtraSize,const BaseBlock *bb)
             // We cannot allocate too much memory here, because above
             // we check FieldSize againt Raw size and we control that Raw size
             // is sensible when reading headers.
-            hd->SubData.Alloc((size_t)FieldSize);
-            Raw->GetB(hd->SubData.Addr(0),(size_t)FieldSize);
+            hd->SubData.resize((size_t)FieldSize);
+            Raw->GetB(hd->SubData.data(),(size_t)FieldSize);
           }
           break;
       }
@@ -1243,7 +1234,7 @@ size_t Archive::ReadHeader14()
     Raw.GetB(Mark,4);
     uint HeadSize=Raw.Get2();
     if (HeadSize<7)
-      return false;
+      return 0;
     byte Flags=Raw.Get1();
     NextBlockPos=CurBlockPos+HeadSize;
     CurHeaderType=HEAD_MAIN;
@@ -1266,7 +1257,7 @@ size_t Archive::ReadHeader14()
     FileHead.FileHash.CRC32=Raw.Get2();
     FileHead.HeadSize=Raw.Get2();
     if (FileHead.HeadSize<21)
-      return false;
+      return 0;
     uint FileTime=Raw.Get4();
     FileHead.FileAttr=Raw.Get1();
     FileHead.Flags=Raw.Get1()|LONG_BLOCK;
@@ -1290,12 +1281,13 @@ size_t Archive::ReadHeader14()
 
     Raw.Read(NameSize);
 
-    char FileName[NM];
-    size_t ReadNameSize=Min(NameSize,ASIZE(FileName)-1);
-    Raw.GetB((byte *)FileName,ReadNameSize);
-    FileName[ReadNameSize]=0;
-    IntToExt(FileName,FileName,ASIZE(FileName));
-    CharToWide(FileName,FileHead.FileName,ASIZE(FileHead.FileName));
+    // RAR 1.4 name size is stored in a single byte field and it can't
+    // exceed 255, so additional checks are not needed.
+    std::string FileName(NameSize,0);
+    Raw.GetB((byte *)&FileName[0],NameSize);
+    std::string NameA;
+    IntToExt(FileName,NameA);
+    CharToWide(NameA,FileHead.FileName);
     ConvertNameCase(FileHead.FileName);
     ConvertFileHeader(&FileHead);
 
@@ -1309,7 +1301,7 @@ size_t Archive::ReadHeader14()
 
 
 #ifndef SFX_MODULE
-void Archive::ConvertNameCase(wchar *Name)
+void Archive::ConvertNameCase(std::wstring &Name)
 {
   if (Cmd->ConvertNames==NAMES_UPPERCASE)
     wcsupper(Name);
@@ -1327,7 +1319,7 @@ bool Archive::IsArcDir()
 
 void Archive::ConvertAttributes()
 {
-#if defined(_WIN_ALL) || defined(_EMX)
+#ifdef _WIN_ALL
   if (FileHead.HSType!=HSYS_WINDOWS)
     FileHead.FileAttr=FileHead.Dir ? 0x10 : 0x20;
 #endif
@@ -1392,19 +1384,23 @@ void Archive::ConvertAttributes()
 
 void Archive::ConvertFileHeader(FileHeader *hd)
 {
+/*
   if (hd->HSType==HSYS_UNKNOWN)
     if (hd->Dir)
       hd->FileAttr=0x10;
     else
       hd->FileAttr=0x20;
+*/
 
 #ifdef _WIN_ALL
   if (hd->HSType==HSYS_UNIX) // Convert Unix, OS X and Android decomposed chracters to Windows precomposed.
-    ConvertToPrecomposed(hd->FileName,ASIZE(hd->FileName));
+    ConvertToPrecomposed(hd->FileName);
 #endif
 
-  for (wchar *s=hd->FileName;*s!=0;s++)
+  for (uint I=0;I<hd->FileName.size();I++)
   {
+    wchar *s=&hd->FileName[I];
+
 #ifdef _UNIX
     // Backslash is the invalid character for Windows file headers,
     // but it can present in Unix file names extracted in Unix.
@@ -1412,7 +1408,7 @@ void Archive::ConvertFileHeader(FileHeader *hd)
       *s='_';
 #endif
 
-#if defined(_WIN_ALL) || defined(_EMX)
+#ifdef _WIN_ALL
     // RAR 5.0 archives do not use '\' as path separator, so if we see it,
     // it means that it is a part of Unix file name, which we cannot
     // extract in Windows.
@@ -1437,6 +1433,9 @@ void Archive::ConvertFileHeader(FileHeader *hd)
     if (*s=='/' || *s=='\\' && Format!=RARFMT50)
       *s=CPATHDIVIDER;
   }
+
+  // Zeroes inside might be possible in broken Unicode names decoded with EncodeFileName::Decode.
+  TruncateAtZero(hd->FileName); // Ensure there are no zeroes inside of string.
 }
 
 
@@ -1451,7 +1450,7 @@ int64 Archive::GetStartPos()
 }
 
 
-bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile,bool TestMode)
+bool Archive::ReadSubData(std::vector<byte> *UnpData,File *DestFile,bool TestMode)
 {
   if (BrokenHeader)
   {
@@ -1459,7 +1458,7 @@ bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile,bool TestMode)
     ErrHandler.SetErrorCode(RARX_CRC);
     return false;
   }
-  if (SubHead.Method>5 || SubHead.UnpVer>(Format==RARFMT50 ? VER_UNPACK5:VER_UNPACK))
+  if (SubHead.Method>5 || SubHead.UnpVer>(Format==RARFMT50 ? VER_UNPACK7:VER_UNPACK))
   {
     uiMsg(UIERROR_SUBHEADERUNKNOWN,FileName);
     return false;
@@ -1486,7 +1485,7 @@ bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile,bool TestMode)
       SubDataIO.SetTestMode(true);
     else
     {
-      UnpData->Alloc((size_t)SubHead.UnpSize);
+      UnpData->resize((size_t)SubHead.UnpSize);
       SubDataIO.SetUnpackToMemory(&(*UnpData)[0],(uint)SubHead.UnpSize);
     }
   }
@@ -1515,7 +1514,7 @@ bool Archive::ReadSubData(Array<byte> *UnpData,File *DestFile,bool TestMode)
     uiMsg(UIERROR_SUBHEADERDATABROKEN,FileName,SubHead.FileName);
     ErrHandler.SetErrorCode(RARX_CRC);
     if (UnpData!=NULL)
-      UnpData->Reset();
+      UnpData->clear();
     return false;
   }
   return true;
