@@ -2,7 +2,7 @@
 #define _RAR_CMDDATA_
 
 
-#define DefaultStoreList L"7z;ace;arj;bz2;cab;gz;jpeg;jpg;lha;lz;lzh;mp3;rar;taz;tgz;xz;z;zip;zipx"
+#define DefaultStoreList L"7z;ace;arj;bz2;cab;gz;jpeg;jpg;lha;lz;lzh;mp3;rar;taz;tbz;tbz2;tgz;txz;xz;z;zip;zipx;zst;tzst"
 
 enum RAR_CMD_LIST_MODE {RCLM_AUTO,RCLM_REJECT_LISTS,RCLM_ACCEPT_LISTS};
 
@@ -18,6 +18,7 @@ class CommandData:public RAROptions
     void SetTimeFilters(const wchar *Mod,bool Before,bool Age);
     void SetStoreTimeMode(const wchar *S);
 #endif
+    int64 GetVolSize(const wchar *S,uint DefMultiplier);
 
     bool FileLists;
     bool NoMoreSwitches;
@@ -28,43 +29,64 @@ class CommandData:public RAROptions
     void Init();
 
     void ParseCommandLine(bool Preprocess,int argc, char *argv[]);
-    void ParseArg(wchar *ArgW);
+    void ParseArg(const wchar *ArgW);
     void ParseDone();
     void ParseEnvVar();
     void ReadConfig();
     void PreprocessArg(const wchar *Arg);
-    void ProcessSwitchesString(const wchar *Str);
+    void ProcessSwitchesString(const std::wstring &Str);
     void OutTitle();
     void OutHelp(RAR_EXIT ExitCode);
     bool IsSwitch(int Ch);
-    bool ExclCheck(const wchar *CheckName,bool Dir,bool CheckFullPath,bool CheckInclList);
-    static bool CheckArgs(StringList *Args,bool Dir,const wchar *CheckName,bool CheckFullPath,int MatchMode);
+    bool ExclCheck(const std::wstring &CheckName,bool Dir,bool CheckFullPath,bool CheckInclList);
+    static bool CheckArgs(StringList *Args,bool Dir,const std::wstring &CheckName,bool CheckFullPath,int MatchMode);
     bool ExclDirByAttr(uint FileAttr);
     bool TimeCheck(RarTime &ftm,RarTime &ftc,RarTime &fta);
     bool SizeCheck(int64 Size);
     bool AnyFiltersActive();
     int IsProcessFile(FileHeader &FileHead,bool *ExactMatch,int MatchType,
-                      bool Flags,wchar *MatchedArg,uint MatchedArgSize);
+                      bool Flags,std::wstring *MatchedArg);
     void ProcessCommand();
-    void AddArcName(const wchar *Name);
+    void AddArcName(const std::wstring &Name);
     bool GetArcName(wchar *Name,int MaxSize);
-    bool CheckWinSize();
+    bool GetArcName(std::wstring &Name);
 
-    int GetRecoverySize(const wchar *CmdStr,const wchar *Value,int DefSize);
 
 #ifndef SFX_MODULE
     void ReportWrongSwitches(RARFORMAT Format);
 #endif
 
-    wchar Command[NM+16];
 
-    wchar ArcName[NM];
+    std::wstring Command;
+    std::wstring ArcName;
+    std::wstring ExtrPath;
+    std::wstring TempPath;
+    std::wstring SFXModule;
+    std::wstring CommentFile;
+    std::wstring ArcPath; // For -ap<path>.
+    std::wstring ExclArcPath; // For -ep4<path> switch.
+    std::wstring LogName;
+    std::wstring EmailTo;
+
+    // Read data from stdin and store in archive under a name specified here
+    // when archiving. Read an archive from stdin if any non-empty string
+    // is specified here when extracting.
+    std::wstring UseStdin;
 
     StringList FileArgs;
     StringList ExclArgs;
     StringList InclArgs;
     StringList ArcNames;
     StringList StoreArgs;
+
+    SecPassword Password;
+
+    std::vector<int64> NextVolSizes;
+
+
+#ifdef RARDLL
+    std::wstring DllDestName;
+#endif
 };
 
 #endif
