@@ -5,10 +5,10 @@ void InitConsole();
 void SetConsoleMsgStream(MESSAGE_TYPE MsgStream);
 void SetConsoleRedirectCharset(RAR_CHARSET RedirectCharset);
 void ProhibitConsoleInput();
-void OutComment(const wchar *Comment,size_t Size);
+void OutComment(const std::wstring &Comment);
 
 #ifndef SILENT
-bool GetConsolePassword(UIPASSWORD_TYPE Type,const wchar *FileName,SecPassword *Password);
+bool GetConsolePassword(UIPASSWORD_TYPE Type,const std::wstring &FileName,SecPassword *Password);
 #endif
 
 #ifdef SILENT
@@ -16,13 +16,13 @@ bool GetConsolePassword(UIPASSWORD_TYPE Type,const wchar *FileName,SecPassword *
   inline void eprintf(const wchar *fmt,...) {}
   inline void Alarm() {}
   inline int Ask(const wchar *AskStr) {return 0;}
-  inline bool getwstr(wchar *str,size_t n) {return false;}
+  inline bool getwstr(std::wstring &str) {return false;}
 #else
   void mprintf(const wchar *fmt,...);
   void eprintf(const wchar *fmt,...);
   void Alarm();
   int Ask(const wchar *AskStr);
-  bool getwstr(wchar *str,size_t n);
+  bool getwstr(std::wstring &str);
 #endif
 
 #endif
