@@ -3,7 +3,6 @@
 File::File()
 {
   hFile=FILE_BAD_HANDLE;
-  *FileName=0;
   NewFile=false;
   LastWrite=false;
   HandleType=FILE_HANDLENORMAL;
@@ -40,12 +39,12 @@ void File::operator = (File &SrcFile)
   LastWrite=SrcFile.LastWrite;
   HandleType=SrcFile.HandleType;
   TruncatedAfterReadError=SrcFile.TruncatedAfterReadError;
-  wcsncpyz(FileName,SrcFile.FileName,ASIZE(FileName));
+  FileName=SrcFile.FileName;
   SrcFile.SkipClose=true;
 }
 
 
-bool File::Open(const wchar *Name,uint Mode)
+bool File::Open(const std::wstring &Name,uint Mode)
 {
   ErrorType=FILE_SUCCESS;
   FileHandle hNewFile;
@@ -63,17 +62,17 @@ bool File::Open(const wchar *Name,uint Mode)
   FindData FD;
   if (PreserveAtime)
     Access|=FILE_WRITE_ATTRIBUTES; // Needed to preserve atime.
-  hNewFile=CreateFile(Name,Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
+  hNewFile=CreateFile(Name.c_str(),Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
 
   DWORD LastError;
   if (hNewFile==FILE_BAD_HANDLE)
   {
     LastError=GetLastError();
 
-    wchar LongName[NM];
-    if (GetWinLongPath(Name,LongName,ASIZE(LongName)))
+    std::wstring LongName;
+    if (GetWinLongPath(Name,LongName))
     {
-      hNewFile=CreateFile(LongName,Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
+      hNewFile=CreateFile(LongName.c_str(),Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
 
       // For archive names longer than 260 characters first CreateFile
       // (without \\?\) fails and sets LastError to 3 (access denied).
@@ -112,10 +111,10 @@ bool File::Open(const wchar *Name,uint Mode)
   if (PreserveAtime)
     flags|=O_NOATIME;
 #endif
-  char NameA[NM];
-  WideToChar(Name,NameA,ASIZE(NameA));
+  std::string NameA;
+  WideToChar(Name,NameA);
 
-  int handle=open(NameA,flags);
+  int handle=open(NameA.c_str(),flags);
 #ifdef LOCK_EX
 
 #ifdef _OSF_SOURCE
@@ -148,7 +147,7 @@ bool File::Open(const wchar *Name,uint Mode)
   if (Success)
   {
     hFile=hNewFile;
-    wcsncpyz(FileName,Name,ASIZE(FileName));
+    FileName=Name;
     TruncatedAfterReadError=false;
   }
   return Success;
@@ -156,7 +155,7 @@ bool File::Open(const wchar *Name,uint Mode)
 
 
 #if !defined(SFX_MODULE)
-void File::TOpen(const wchar *Name)
+void File::TOpen(const std::wstring &Name)
 {
   if (!WOpen(Name))
     ErrHandler.Exit(RARX_OPEN);
@@ -164,7 +163,7 @@ void File::TOpen(const wchar *Name)
 #endif
 
 
-bool File::WOpen(const wchar *Name)
+bool File::WOpen(const std::wstring &Name)
 {
   if (Open(Name))
     return true;
@@ -173,7 +172,7 @@ bool File::WOpen(const wchar *Name)
 }
 
 
-bool File::Create(const wchar *Name,uint Mode)
+bool File::Create(const std::wstring &Name,uint Mode)
 {
   // OpenIndiana based NAS and CIFS shares fail to set the file time if file
   // was created in read+write mode and some data was written and not flushed
@@ -188,40 +187,40 @@ bool File::Create(const wchar *Name,uint Mode)
 
   // Windows automatically removes dots and spaces in the end of file name,
   // So we detect such names and process them with \\?\ prefix.
-  wchar *LastChar=PointToLastChar(Name);
-  bool Special=*LastChar=='.' || *LastChar==' ';
+  wchar LastChar=GetLastChar(Name);
+  bool Special=LastChar=='.' || LastChar==' ';
   
   if (Special && (Mode & FMF_STANDARDNAMES)==0)
     hFile=FILE_BAD_HANDLE;
   else
-    hFile=CreateFile(Name,Access,ShareMode,NULL,CREATE_ALWAYS,0,NULL);
+    hFile=CreateFile(Name.c_str(),Access,ShareMode,NULL,CREATE_ALWAYS,0,NULL);
 
   if (hFile==FILE_BAD_HANDLE)
   {
-    wchar LongName[NM];
-    if (GetWinLongPath(Name,LongName,ASIZE(LongName)))
-      hFile=CreateFile(LongName,Access,ShareMode,NULL,CREATE_ALWAYS,0,NULL);
+    std::wstring LongName;
+    if (GetWinLongPath(Name,LongName))
+      hFile=CreateFile(LongName.c_str(),Access,ShareMode,NULL,CREATE_ALWAYS,0,NULL);
   }
 
 #else
-  char NameA[NM];
-  WideToChar(Name,NameA,ASIZE(NameA));
 #ifdef FILE_USE_OPEN
-  hFile=open(NameA,(O_CREAT|O_TRUNC) | (WriteMode ? O_WRONLY : O_RDWR),0666);
+  std::string NameA;
+  WideToChar(Name,NameA);
+  hFile=open(NameA.c_str(),(O_CREAT|O_TRUNC) | (WriteMode ? O_WRONLY : O_RDWR),0666);
 #else
-  hFile=fopen(NameA,WriteMode ? WRITEBINARY:CREATEBINARY);
+  hFile=fopen(NameA.c_str(),WriteMode ? WRITEBINARY:CREATEBINARY);
 #endif
 #endif
   NewFile=true;
   HandleType=FILE_HANDLENORMAL;
   SkipClose=false;
-  wcsncpyz(FileName,Name,ASIZE(FileName));
+  FileName=Name;
   return hFile!=FILE_BAD_HANDLE;
 }
 
 
 #if !defined(SFX_MODULE)
-void File::TCreate(const wchar *Name,uint Mode)
+void File::TCreate(const std::wstring &Name,uint Mode)
 {
   if (!WCreate(Name,Mode))
     ErrHandler.Exit(RARX_FATAL);
@@ -229,7 +228,7 @@ void File::TCreate(const wchar *Name,uint Mode)
 #endif
 
 
-bool File::WCreate(const wchar *Name,uint Mode)
+bool File::WCreate(const std::wstring &Name,uint Mode)
 {
   if (Create(Name,Mode))
     return true;
@@ -250,7 +249,7 @@ bool File::Close()
       // We use the standard system handle for stdout in Windows
       // and it must not be closed here.
       if (HandleType==FILE_HANDLENORMAL)
-        Success=CloseHandle(hFile)==TRUE;
+        Success=CloseHandle(hFile)!=FALSE;
 #else
 #ifdef FILE_USE_OPEN
       Success=close(hFile)!=-1;
@@ -280,16 +279,16 @@ bool File::Delete()
 }
 
 
-bool File::Rename(const wchar *NewName)
+bool File::Rename(const std::wstring &NewName)
 {
   // No need to rename if names are already same.
-  bool Success=wcscmp(FileName,NewName)==0;
+  bool Success=(NewName==FileName);
 
   if (!Success)
     Success=RenameFile(FileName,NewName);
 
   if (Success)
-    wcsncpyz(FileName,NewName,ASIZE(FileName));
+    FileName=NewName;
 
   return Success;
 }
@@ -327,13 +326,13 @@ bool File::Write(const void *Data,size_t Size)
       const size_t MaxSize=0x4000;
       for (size_t I=0;I<Size;I+=MaxSize)
       {
-        Success=WriteFile(hFile,(byte *)Data+I,(DWORD)Min(Size-I,MaxSize),&Written,NULL)==TRUE;
+        Success=WriteFile(hFile,(byte *)Data+I,(DWORD)Min(Size-I,MaxSize),&Written,NULL)!=FALSE;
         if (!Success)
           break;
       }
     }
     else
-      Success=WriteFile(hFile,Data,(DWORD)Size,&Written,NULL)==TRUE;
+      Success=WriteFile(hFile,Data,(DWORD)Size,&Written,NULL)!=FALSE;
 #else
 #ifdef FILE_USE_OPEN
     ssize_t Written=write(hFile,Data,Size);
@@ -362,7 +361,7 @@ bool File::Write(const void *Data,size_t Size)
           Seek(Tell()-Written,SEEK_SET);
         continue;
       }
-      ErrHandler.WriteError(NULL,FileName);
+      ErrHandler.WriteError(L"",FileName);
     }
     break;
   }
@@ -405,7 +404,7 @@ int File::Read(void *Data,size_t Size)
         else
         {
           bool Ignore=false,Retry=false,Quit=false;
-          if (ReadErrorMode==FREM_ASK && HandleType==FILE_HANDLENORMAL)
+          if (ReadErrorMode==FREM_ASK && HandleType==FILE_HANDLENORMAL && IsOpened())
           {
             ErrHandler.AskRepeatRead(FileName,Ignore,Retry,Quit);
             if (Retry)
@@ -522,29 +521,35 @@ bool File::RawSeek(int64 Offset,int Method)
 {
   if (hFile==FILE_BAD_HANDLE)
     return true;
-  if (!IsSeekable())
+  if (!IsSeekable()) // To extract archives from stdin with -si.
   {
-    if (Method==SEEK_CUR)
+    // We tried to dynamically allocate 32 KB buffer here, but it improved
+    // speed in Windows 10 by mere ~1.5%.
+    byte Buf[4096];
+    if (Method==SEEK_CUR || Method==SEEK_SET && Offset>=CurFilePos)
     {
-      Offset+=CurFilePos;
-      Method=SEEK_SET;
-    }
-    if (Method==SEEK_SET && Offset>=CurFilePos) // Reading for seek forward.
-    {
-      uint64 SkipSize=Offset-CurFilePos;
-      while (SkipSize>0)
+      uint64 SkipSize=Method==SEEK_CUR ? Offset:Offset-CurFilePos;
+      while (SkipSize>0) // Reading to emulate seek forward.
       {
-        byte Buf[4096];
         int ReadSize=Read(Buf,(size_t)Min(SkipSize,ASIZE(Buf)));
         if (ReadSize<=0)
           return false;
         SkipSize-=ReadSize;
+        CurFilePos+=ReadSize;
       }
-      CurFilePos=Offset;
+      return true;
+    }
+    // May need it in FileLength() in Archive::UnexpEndArcMsg() when unpacking
+    // RAR 4.x archives without the end of archive block created with -en.
+    if (Method==SEEK_END)
+    {
+      int ReadSize;
+      while ((ReadSize=Read(Buf,ASIZE(Buf)))>0)
+        CurFilePos+=ReadSize;
       return true;
     }
 
-    return false; // Backward or end of file seek on unseekable file.
+    return false; // Backward seek on unseekable file.
   }
   if (Offset<0 && Method!=SEEK_SET)
   {
@@ -640,7 +645,7 @@ void File::PutByte(byte Byte)
 bool File::Truncate()
 {
 #ifdef _WIN_ALL
-  return SetEndOfFile(hFile)==TRUE;
+  return SetEndOfFile(hFile)!=FALSE;
 #else
   return ftruncate(GetFD(),(off_t)Tell())==0;
 #endif
@@ -698,15 +703,15 @@ void File::SetCloseFileTime(RarTime *ftm,RarTime *fta)
 }
 
 
-void File::SetCloseFileTimeByName(const wchar *Name,RarTime *ftm,RarTime *fta)
+void File::SetCloseFileTimeByName(const std::wstring &Name,RarTime *ftm,RarTime *fta)
 {
 #ifdef _UNIX
   bool setm=ftm!=NULL && ftm->IsSet();
   bool seta=fta!=NULL && fta->IsSet();
   if (setm || seta)
   {
-    char NameA[NM];
-    WideToChar(Name,NameA,ASIZE(NameA));
+    std::string NameA;
+    WideToChar(Name,NameA);
 
 #ifdef UNIX_TIME_NS
     timespec times[2];
@@ -714,7 +719,7 @@ void File::SetCloseFileTimeByName(const wchar *Name,RarTime *ftm,RarTime *fta)
     times[0].tv_nsec=seta ? long(fta->GetUnixNS()%1000000000) : UTIME_NOW;
     times[1].tv_sec=setm ? ftm->GetUnix() : 0;
     times[1].tv_nsec=setm ? long(ftm->GetUnixNS()%1000000000) : UTIME_NOW;
-    utimensat(AT_FDCWD,NameA,times,0);
+    utimensat(AT_FDCWD,NameA.c_str(),times,0);
 #else
     utimbuf ut;
     if (setm)
@@ -725,24 +730,47 @@ void File::SetCloseFileTimeByName(const wchar *Name,RarTime *ftm,RarTime *fta)
       ut.actime=fta->GetUnix();
     else
       ut.actime=ut.modtime; // Need to set something, cannot left it 0.
-    utime(NameA,&ut);
+    utime(NameA.c_str(),&ut);
 #endif
   }
 #endif
 }
 
 
-void File::GetOpenFileTime(RarTime *ft)
+#ifdef _UNIX
+void File::StatToRarTime(struct stat &st,RarTime *ftm,RarTime *ftc,RarTime *fta)
+{
+#ifdef UNIX_TIME_NS
+#if defined(_APPLE)
+  if (ftm!=NULL) ftm->SetUnixNS(st.st_mtimespec.tv_sec*(uint64)1000000000+st.st_mtimespec.tv_nsec);
+  if (ftc!=NULL) ftc->SetUnixNS(st.st_ctimespec.tv_sec*(uint64)1000000000+st.st_ctimespec.tv_nsec);
+  if (fta!=NULL) fta->SetUnixNS(st.st_atimespec.tv_sec*(uint64)1000000000+st.st_atimespec.tv_nsec);
+#else
+  if (ftm!=NULL) ftm->SetUnixNS(st.st_mtim.tv_sec*(uint64)1000000000+st.st_mtim.tv_nsec);
+  if (ftc!=NULL) ftc->SetUnixNS(st.st_ctim.tv_sec*(uint64)1000000000+st.st_ctim.tv_nsec);
+  if (fta!=NULL) fta->SetUnixNS(st.st_atim.tv_sec*(uint64)1000000000+st.st_atim.tv_nsec);
+#endif
+#else
+  if (ftm!=NULL) ftm->SetUnix(st.st_mtime);
+  if (ftc!=NULL) ftc->SetUnix(st.st_ctime);
+  if (fta!=NULL) fta->SetUnix(st.st_atime);
+#endif
+}
+#endif
+
+
+void File::GetOpenFileTime(RarTime *ftm,RarTime *ftc,RarTime *fta)
 {
 #ifdef _WIN_ALL
-  FILETIME FileTime;
-  GetFileTime(hFile,NULL,NULL,&FileTime);
-  ft->SetWinFT(&FileTime);
-#endif
-#if defined(_UNIX) || defined(_EMX)
+  FILETIME ctime,atime,mtime;
+  GetFileTime(hFile,&ctime,&atime,&mtime);
+  if (ftm!=NULL) ftm->SetWinFT(&mtime);
+  if (ftc!=NULL) ftc->SetWinFT(&ctime);
+  if (fta!=NULL) fta->SetWinFT(&atime);
+#elif defined(_UNIX)
   struct stat st;
   fstat(GetFD(),&st);
-  ft->SetUnix(st.st_mtime);
+  StatToRarTime(st,ftm,ftc,fta);
 #endif
 }
 
@@ -773,15 +801,15 @@ bool File::IsDevice()
 #ifndef SFX_MODULE
 int64 File::Copy(File &Dest,int64 Length)
 {
-  Array<byte> Buffer(File::CopyBufferSize());
+  std::vector<byte> Buffer(File::CopyBufferSize());
   int64 CopySize=0;
   bool CopyAll=(Length==INT64NDF);
 
   while (CopyAll || Length>0)
   {
     Wait();
-    size_t SizeToRead=(!CopyAll && Length<(int64)Buffer.Size()) ? (size_t)Length:Buffer.Size();
-    byte *Buf=&Buffer[0];
+    size_t SizeToRead=(!CopyAll && Length<(int64)Buffer.size()) ? (size_t)Length:Buffer.size();
+    byte *Buf=Buffer.data();
     int ReadSize=Read(Buf,SizeToRead);
     if (ReadSize==0)
       break;
