@@ -32,7 +32,24 @@ class DataHash;
 
 class DataHash
 {
+  public:
+    struct CRC32ThreadData
+    {
+      void *Data;
+      size_t DataSize;
+      uint DataCRC;
+    };
   private:
+    void UpdateCRC32MT(const void *Data,size_t DataSize);
+    uint BitReverse32(uint N);
+    uint gfMulCRC(uint A, uint B);
+    uint gfExpCRC(uint N);
+
+    // Speed gain seems to vanish above 8 CRC32 threads.
+    static const uint CRC32_POOL_THREADS=8;
+    // Thread pool must allow at least BLAKE2_THREADS_NUMBER threads.
+    static const uint HASH_POOL_THREADS=Max(BLAKE2_THREADS_NUMBER,CRC32_POOL_THREADS);
+
     HASH_TYPE HashType;
     uint CurCRC32;
     blake2sp_state *blake2ctx;
@@ -41,8 +58,6 @@ class DataHash
     ThreadPool *ThPool;
 
     uint MaxThreads;
-    // Upper limit for maximum threads to prevent wasting threads in pool.
-    static const uint MaxHashThreads=8;
 #endif
   public:
     DataHash();
