@@ -62,30 +62,24 @@ static bool WMI_IsWindows10()
   hres = pSvc->ExecQuery(bstr_t("WQL"), bstr_t("SELECT * FROM Win32_OperatingSystem"),
          WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, NULL, &pEnumerator);
     
-  if (FAILED(hres))
+  if (FAILED(hres) || pEnumerator==NULL)
   {
     pSvc->Release();
     pLoc->Release();
     return false;
   }
 
+  bool Win10=false;
+
   IWbemClassObject *pclsObj = NULL;
   ULONG uReturn = 0;
-   
-  bool Win10=false;
-  while (pEnumerator!=NULL)
+  pEnumerator->Next(WBEM_INFINITE, 1, &pclsObj, &uReturn);
+  if (pclsObj!=NULL && uReturn>0)
   {
-    HRESULT hr = pEnumerator->Next(WBEM_INFINITE, 1, &pclsObj, &uReturn);
-
-    if (uReturn==0)
-      break;
-
     VARIANT vtProp;
-
-    hr = pclsObj->Get(L"Name", 0, &vtProp, 0, 0);
+    pclsObj->Get(L"Name", 0, &vtProp, 0, 0);
     Win10|=wcsstr(vtProp.bstrVal,L"Windows 10")!=NULL;
     VariantClear(&vtProp);
-
     pclsObj->Release();
   }
 
