@@ -45,13 +45,25 @@ enum OVERWRITE_MODE
   OVERWRITE_FORCE_ASK
 };
 
+enum ARC_METADATA
+{
+  ARCMETA_NONE=0,
+  ARCMETA_SAVE,    // -ams
+  ARCMETA_RESTORE  // -amr
+};
 
 enum QOPEN_MODE { QOPEN_NONE, QOPEN_AUTO, QOPEN_ALWAYS };
 
 enum RAR_CHARSET { RCH_DEFAULT=0,RCH_ANSI,RCH_OEM,RCH_UNICODE,RCH_UTF8 };
 
 #define     MAX_FILTER_TYPES           16
-enum FilterState {FILTER_DEFAULT=0,FILTER_AUTO,FILTER_FORCE,FILTER_DISABLE};
+
+enum FilterState {
+  FILTER_DEFAULT=0, // No -mc<filter> switch.
+  FILTER_AUTO,      // -mc<filter> switch is present.
+  FILTER_FORCE,     // -mc<filter>+ switch is present.
+  FILTER_DISABLE    // -mc<filter>- switch is present.
+};
 
 
 enum SAVECOPY_MODE {
@@ -84,13 +96,15 @@ struct FilterMode
 #define MAX_GENERATE_MASK  128
 
 
+// Here we store simple data types, which we can clear and move all together
+// quickly. Rest of data types goes to CommandData.
 class RAROptions
 {
   public:
     RAROptions();
-    ~RAROptions();
     void Init();
 
+
     uint ExclFileAttr;
     uint InclFileAttr;
 
@@ -100,31 +114,24 @@ class RAROptions
     bool InclDir;
 
     bool InclAttrSet;
-    size_t WinSize;
-    wchar TempPath[NM];
-    wchar SFXModule[NM];
+    uint64 WinSize;
+    uint64 WinSizeLimit; // Switch -mdx<size>.
 
 #ifdef USE_QOPEN
     QOPEN_MODE QOpenMode;
 #endif
 
     bool ConfigDisabled; // Switch -cfg-.
-    wchar ExtrPath[NM];
-    wchar CommentFile[NM];
     RAR_CHARSET CommentCharset;
     RAR_CHARSET FilelistCharset;
     RAR_CHARSET ErrlogCharset;
     RAR_CHARSET RedirectCharset;
 
-    wchar ArcPath[NM]; // For -ap<path>.
-    wchar ExclArcPath[NM]; // For -ep4<path> switch.
-    SecPassword Password;
     bool EncryptHeaders;
     bool SkipEncrypted;
     
     bool ManualPassword; // Password entered manually during operation, might need to clean for next archive.
 
-    wchar LogName[NM];
     MESSAGE_TYPE MsgStream;
     SOUND_NOTIFY_MODE Sound;
     OVERWRITE_MODE Overwrite;
@@ -132,6 +139,7 @@ class RAROptions
     HASH_TYPE HashType;
     int Recovery;
     int RecVolNumber;
+    ARC_METADATA ArcMetadata;
     bool DisablePercentage;
     bool DisableCopyright;
     bool DisableDone;
@@ -147,7 +155,6 @@ class RAROptions
     PATH_EXCL_MODE ExclPath;
     RECURSE_MODE Recurse;
     int64 VolSize;
-    Array<int64> NextVolSizes;
     uint CurVolNum;
     bool AllYes;
     bool VerboseOutput; // -iv, display verbose output, used only in "WinRAR t" now.
@@ -158,6 +165,7 @@ class RAROptions
     bool SaveSymLinks;
     bool SaveHardLinks;
     bool AbsoluteLinks;
+    bool SkipSymLinks;
     int Priority;
     int SleepTime;
     bool KeepBroken;
@@ -189,7 +197,6 @@ class RAROptions
     bool Test;
     bool VolumePause;
     FilterMode FilterModes[MAX_FILTER_TYPES];
-    wchar EmailTo[NM];
     uint VersionControl;
     APPENDARCNAME_MODE AppendArcNameToPath;
     POWER_MODE Shutdown;
@@ -198,11 +205,6 @@ class RAROptions
     EXTTIME_MODE xatime;
     bool PreserveAtime;
 
-    // Read data from stdin and store in archive under a name specified here
-    // when archiving. Read an archive from stdin if any non-empty string
-    // is specified here when extracting.
-    wchar UseStdin[NM];
-
     uint Threads; // We use it to init hash even if RAR_SMP is not defined.
 
 
@@ -210,7 +212,6 @@ class RAROptions
 
 
 #ifdef RARDLL
-    wchar DllDestName[NM];
     int DllOpMode;
     int DllError;
     LPARAM UserData;
@@ -218,5 +219,6 @@ class RAROptions
     CHANGEVOLPROC ChangeVolProc;
     PROCESSDATAPROC ProcessDataProc;
 #endif
+
 };
 #endif
