@@ -4,24 +4,20 @@
 #define FALSE 0
 #define TRUE  1
 
-#ifdef __EMX__
-  #define INCL_BASE
-#endif
-
 #if defined(RARDLL) && !defined(SILENT)
 #define SILENT
 #endif
 
 #include <new>
+#include <string>
+#include <vector>
+#include <memory> // For automatic pointers.
 
 
-#if defined(_WIN_ALL) || defined(_EMX)
-
-#define LITTLE_ENDIAN
-#define NM  2048
-
 #ifdef _WIN_ALL
 
+#define LITTLE_ENDIAN
+
 
 // We got a report that just "#define STRICT" is incompatible with
 // "#define STRICT 1" in Windows 10 SDK minwindef.h and depending on the order
@@ -61,31 +57,23 @@
 #include <wchar.h>
 #include <wctype.h>
 
+// For WMI requests.
+#include <comdef.h>
+#include <Wbemidl.h>
+#pragma comment(lib, "wbemuuid.lib")
 
-#endif // _WIN_ALL
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dos.h>
+#include <direct.h>
+#include <intrin.h>
 
-#if !defined(_EMX) && !defined(_MSC_VER)
-  #include <dir.h>
+// Use SSE only for x86/x64, not ARM Windows.
+#if defined(_M_IX86) || defined(_M_X64)
+  #define USE_SSE
+  #define SSE_ALIGNMENT 16
 #endif
-#ifdef _MSC_VER
-  #if _MSC_VER<1500
-    #define for if (0) ; else for
-  #endif
-  #include <direct.h>
-  #include <intrin.h>
-
-  // Use SSE only for x86/x64, not ARM Windows.
-  #if defined(_M_IX86) || defined(_M_X64)
-    #define USE_SSE
-    #define SSE_ALIGNMENT 16
-  #endif
-#else
-  #include <dirent.h>
-#endif // _MSC_VER
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -98,7 +86,6 @@
 #include <time.h>
 #include <signal.h>
 
-
 #define SAVE_LINKS
 
 #define ENABLE_ACCESS
@@ -108,7 +95,7 @@
 
 
 #define SPATHDIVIDER L"\\"
-#define CPATHDIVIDER '\\'
+#define CPATHDIVIDER L'\\'
 #define MASKALL      L"*"
 
 #define READBINARY   "rb"
@@ -118,25 +105,13 @@
 #define WRITEBINARY  "wb"
 #define APPENDTEXT   "at"
 
-#if defined(_WIN_ALL)
-  #ifdef _MSC_VER
-    #define _stdfunction __cdecl
-    #define _forceinline __forceinline
-  #else
-    #define _stdfunction _USERENTRY
-    #define _forceinline inline
-  #endif
-#else
-  #define _stdfunction
-  #define _forceinline inline
-#endif
+#define _stdfunction __cdecl
+#define _forceinline __forceinline
 
-#endif // defined(_WIN_ALL) || defined(_EMX)
+#endif // _WIN_ALL
 
 #ifdef _UNIX
 
-#define NM  2048
-
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -144,7 +119,7 @@
 #if defined(__QNXNTO__)
   #include <sys/param.h>
 #endif
-#if defined(RAR_SMP) && defined(__APPLE__)
+#ifdef _APPLE
   #include <sys/sysctl.h>
 #endif
 #ifndef SFX_MODULE
@@ -167,6 +142,28 @@
 #include <utime.h>
 #include <locale.h>
 
+#ifdef __GNUC__
+  #if defined(__i386__) || defined(__x86_64__)
+    #include <x86intrin.h>
+    
+    #define USE_SSE
+    #define SSE_ALIGNMENT 16
+  #endif
+#endif
+
+#if defined(__aarch64__) && (defined(__ARM_FEATURE_CRYPTO) || defined(__ARM_FEATURE_CRC32))
+#include <arm_neon.h>
+#ifndef _APPLE
+#include <sys/auxv.h>
+#include <asm/hwcap.h>
+#endif
+#ifdef __ARM_FEATURE_CRYPTO
+#define USE_NEON_AES
+#endif
+#ifdef __ARM_FEATURE_CRC32
+#define USE_NEON_CRC32
+#endif
+#endif
 
 #ifdef  S_IFLNK
 #define SAVE_LINKS
@@ -184,7 +181,7 @@
 
 
 #define SPATHDIVIDER L"/"
-#define CPATHDIVIDER '/'
+#define CPATHDIVIDER L'/'
 #define MASKALL      L"*"
 
 #define READBINARY   "r"
@@ -221,19 +218,12 @@
 
 #endif // _UNIX
 
-#if 0
-  #define MSGID_INT
-  typedef int MSGID;
-#else
   typedef const wchar* MSGID;
-#endif
 
 #ifndef SSE_ALIGNMENT // No SSE use and no special data alignment is required.
   #define SSE_ALIGNMENT 1
 #endif
 
-#define safebuf static
-
 // Solaris defines _LITTLE_ENDIAN or _BIG_ENDIAN.
 #if defined(_LITTLE_ENDIAN) && !defined(LITTLE_ENDIAN)
   #define LITTLE_ENDIAN
@@ -264,8 +254,8 @@
   #endif
 #endif
 
-#if !defined(BIG_ENDIAN) && defined(_WIN_ALL) || defined(__i386__) || defined(__x86_64__)
-// Allow not aligned integer access, increases speed in some operations.
+#if !defined(BIG_ENDIAN) && defined(_WIN_ALL) || defined(__i386__) || defined(__x86_64__) || defined(__aarch64__)
+// Allow unaligned integer access, increases speed in some operations.
 #define ALLOW_MISALIGNED
 #endif
 
