@@ -155,7 +155,7 @@ void ComprDataIO::UnpWrite(byte *Addr,size_t Count)
 {
 
 #ifdef RARDLL
-  RAROptions *Cmd=((Archive *)SrcFile)->GetRAROptions();
+  CommandData *Cmd=((Archive *)SrcFile)->GetCommandData();
   if (Cmd->DllOpMode!=RAR_SKIP)
   {
     if (Cmd->Callback!=NULL &&
@@ -204,7 +204,7 @@ void ComprDataIO::ShowUnpRead(int64 ArcPos,int64 ArcSize)
     ArcPos+=ProcessedArcSize;
 
     Archive *SrcArc=(Archive *)SrcFile;
-    RAROptions *Cmd=SrcArc->GetRAROptions();
+    CommandData *Cmd=SrcArc->GetCommandData();
 
     int CurPercent=ToPercent(ArcPos,ArcSize);
     if (!Cmd->DisablePercentage && CurPercent!=LastPercent)
@@ -259,15 +259,6 @@ void ComprDataIO::SetEncryption(bool Encrypt,CRYPT_METHOD Method,
 }
 
 
-#if !defined(SFX_MODULE) && !defined(RAR_NOCRYPT)
-void ComprDataIO::SetAV15Encryption()
-{
-  Decryption=true;
-  Decrypt->SetAV15Encryption();
-}
-#endif
-
-
 #if !defined(SFX_MODULE) && !defined(RAR_NOCRYPT)
 void ComprDataIO::SetCmt13Encryption()
 {
@@ -299,11 +290,15 @@ void ComprDataIO::AdjustTotalArcSize(Archive *Arc)
   // packed size to beginning of these blocks. Earlier we already calculated
   // the total size based on entire archive sizes. We also set LastArcSize
   // to start of first trailing block, to add it later to ProcessedArcSize.
-  int64 ArcLength=Arc->IsSeekable() ? Arc->FileLength() : 0;
-  if (Arc->MainHead.QOpenOffset!=0) // QO is always preceding RR record.
+  uint64 ArcLength=Arc->IsSeekable() ? Arc->FileLength() : 0;
+  // QO is always preceding RR record.
+  // Also we check QO and RR to be less than archive length to prevent
+  // negative "ArcLength-LastArcSize" and possible signed integer overflow
+  // when calculating TotalArcSize.
+  if (Arc->MainHead.QOpenOffset>0 && Arc->MainHead.QOpenOffset<ArcLength)
     LastArcSize=Arc->MainHead.QOpenOffset;
   else
-    if (Arc->MainHead.RROffset!=0)
+    if (Arc->MainHead.RROffset>0 && Arc->MainHead.RROffset<ArcLength)
       LastArcSize=Arc->MainHead.RROffset;
     else
     {
