@@ -1,14 +1,11 @@
 #ifndef _RAR_DATAIO_
 #define _RAR_DATAIO_
 
+class Archive;
 class CmdAdd;
 class Unpack;
 class ArcFileSearch;
 
-#if 0
-// We use external i/o calls for Benchmark command.
-#define COMPRDATAIO_EXTIO
-#endif
 
 class ComprDataIO
 {
@@ -29,6 +26,7 @@ class ComprDataIO
     byte *UnpWrAddr;
 
     int64 UnpPackedSize;
+    int64 UnpPackedLeft;
 
     bool ShowProgress;
     bool TestMode;
@@ -61,7 +59,7 @@ class ComprDataIO
     void UnpWrite(byte *Addr,size_t Count);
     void EnableShowProgress(bool Show) {ShowProgress=Show;}
     void GetUnpackedData(byte **Data,size_t *Size);
-    void SetPackedSizeToRead(int64 Size) {UnpPackedSize=Size;}
+    void SetPackedSizeToRead(int64 Size) {UnpPackedSize=UnpPackedLeft=Size;}
     void SetTestMode(bool Mode) {TestMode=Mode;}
     void SetSkipUnpCRC(bool Skip) {SkipUnpCRC=Skip;}
     void SetNoFileHeader(bool Mode) {NoFileHeader=Mode;}
@@ -70,16 +68,15 @@ class ComprDataIO
     void SetSubHeader(FileHeader *hd,int64 *Pos) {SubHead=hd;SubHeadPos=Pos;}
     void SetEncryption(bool Encrypt,CRYPT_METHOD Method,SecPassword *Password,
          const byte *Salt,const byte *InitV,uint Lg2Cnt,byte *HashKey,byte *PswCheck);
-    void SetAV15Encryption();
     void SetCmt13Encryption();
     void SetUnpackToMemory(byte *Addr,uint Size);
     void SetCurrentCommand(wchar Cmd) {CurrentCommand=Cmd;}
+    void AdjustTotalArcSize(Archive *Arc);
 
 
     bool PackVolume;
     bool UnpVolume;
     bool NextVolumeMissing;
-    int64 UnpArcSize;
     int64 CurPackRead,CurPackWrite,CurUnpRead,CurUnpWrite;
 
 
@@ -87,6 +84,9 @@ class ComprDataIO
     // Used to calculate the total operation progress.
     int64 ProcessedArcSize;
 
+    // Last extracted archive size up to QO or RR block.
+    int64 LastArcSize;
+
     int64 TotalArcSize;
 
     DataHash PackedDataHash; // Packed write and unpack read hash.
