@@ -102,7 +102,7 @@ void Wait()
 
 
 #if defined(_WIN_ALL) && !defined(SFX_MODULE)
-void Shutdown()
+void Shutdown(POWER_MODE Mode)
 {
   HANDLE hToken;
   TOKEN_PRIVILEGES tkp;
@@ -114,23 +114,54 @@ void Shutdown()
 
     AdjustTokenPrivileges(hToken,FALSE,&tkp,0,(PTOKEN_PRIVILEGES)NULL,0);
   }
-  ExitWindowsEx(EWX_SHUTDOWN|EWX_FORCE|EWX_POWEROFF,SHTDN_REASON_FLAG_PLANNED);
+  if (Mode==POWERMODE_OFF)
+    ExitWindowsEx(EWX_SHUTDOWN|EWX_FORCE,SHTDN_REASON_FLAG_PLANNED);
+  if (Mode==POWERMODE_SLEEP)
+    SetSuspendState(FALSE,FALSE,FALSE);
+  if (Mode==POWERMODE_HIBERNATE)
+    SetSuspendState(TRUE,FALSE,FALSE);
+  if (Mode==POWERMODE_RESTART)
+    ExitWindowsEx(EWX_REBOOT|EWX_FORCE,SHTDN_REASON_FLAG_PLANNED);
+}
+
+
+bool ShutdownCheckAnother(bool Open)
+{
+  const wchar *EventName=L"rar -ioff";
+  static HANDLE hEvent=NULL;
+  bool Result=false; // Return false if no other RAR -ioff are running.
+  if (Open) // Create or open the event.
+    hEvent=CreateEvent(NULL,FALSE,FALSE,EventName);
+  else
+  {
+    if (hEvent!=NULL)
+      CloseHandle(hEvent); // Close our event.
+    // Check if other copies still own the event. While race conditions
+    // are possible, they are improbable and their harm is minimal.
+    hEvent=CreateEvent(NULL,FALSE,FALSE,EventName);
+    Result=GetLastError()==ERROR_ALREADY_EXISTS;
+    if (hEvent!=NULL)
+      CloseHandle(hEvent);
+  }
+  return Result;
 }
 #endif
 
 
 
 
+
 #if defined(_WIN_ALL)
 // Load library from Windows System32 folder. Use this function to prevent
 // loading a malicious code from current folder or same folder as exe.
 HMODULE WINAPI LoadSysLibrary(const wchar *Name)
 {
-  wchar SysDir[NM];
-  if (GetSystemDirectory(SysDir,ASIZE(SysDir))==0)
-    return NULL;
-  MakeName(SysDir,Name,SysDir,ASIZE(SysDir));
-  return LoadLibrary(SysDir);
+  std::vector<wchar> SysDir(MAX_PATH);
+  if (GetSystemDirectory(SysDir.data(),(UINT)SysDir.size())==0)
+    return nullptr;
+  std::wstring FullName;
+  MakeName(SysDir.data(),Name,FullName);
+  return LoadLibrary(FullName.c_str());
 }
 
 
@@ -157,19 +188,43 @@ SSE_VERSION _SSE_Version=GetSSEVersion();
 
 SSE_VERSION GetSSEVersion()
 {
+#ifdef _MSC_VER
   int CPUInfo[4];
-  __cpuid(CPUInfo, 7);
-  if ((CPUInfo[1] & 0x20)!=0)
+  __cpuid(CPUInfo, 0);
+
+  // Maximum supported cpuid function.
+  uint MaxSupported=CPUInfo[0];
+
+  if (MaxSupported>=7)
+  {
+    __cpuid(CPUInfo, 7);
+    if ((CPUInfo[1] & 0x20)!=0)
+      return SSE_AVX2;
+  }
+  if (MaxSupported>=1)
+  {
+    __cpuid(CPUInfo, 1);
+    if ((CPUInfo[2] & 0x80000)!=0)
+      return SSE_SSE41;
+    if ((CPUInfo[2] & 0x200)!=0)
+      return SSE_SSSE3;
+    if ((CPUInfo[3] & 0x4000000)!=0)
+      return SSE_SSE2;
+    if ((CPUInfo[3] & 0x2000000)!=0)
+      return SSE_SSE;
+  }
+#elif defined(__GNUC__)
+  if (__builtin_cpu_supports("avx2"))
     return SSE_AVX2;
-  __cpuid(CPUInfo, 1);
-  if ((CPUInfo[2] & 0x80000)!=0)
+  if (__builtin_cpu_supports("sse4.1"))
     return SSE_SSE41;
-  if ((CPUInfo[2] & 0x200)!=0)
+  if (__builtin_cpu_supports("ssse3"))
     return SSE_SSSE3;
-  if ((CPUInfo[3] & 0x4000000)!=0)
+  if (__builtin_cpu_supports("sse2"))
     return SSE_SSE2;
-  if ((CPUInfo[3] & 0x2000000)!=0)
+  if (__builtin_cpu_supports("sse"))
     return SSE_SSE;
+#endif
   return SSE_NONE;
 }
 #endif
