@@ -71,12 +71,12 @@ void RarTime::SetLocal(RarLocalTime *lt)
 {
 #ifdef _WIN_ALL
   SYSTEMTIME st;
-  st.wYear=lt->Year;
-  st.wMonth=lt->Month;
-  st.wDay=lt->Day;
-  st.wHour=lt->Hour;
-  st.wMinute=lt->Minute;
-  st.wSecond=lt->Second;
+  st.wYear=(WORD)lt->Year;
+  st.wMonth=(WORD)lt->Month;
+  st.wDay=(WORD)lt->Day;
+  st.wHour=(WORD)lt->Hour;
+  st.wMinute=(WORD)lt->Minute;
+  st.wSecond=(WORD)lt->Second;
   st.wMilliseconds=0;
   st.wDayOfWeek=0;
   FILETIME lft;
@@ -183,8 +183,7 @@ void RarTime::SetUnix(time_t ut)
 // Get the high precision Unix time in nanoseconds since 01-01-1970.
 uint64 RarTime::GetUnixNS()
 {
-  // 11644473600000000000 - number of ns between 01-01-1601 and 01-01-1970.
-  uint64 ushift=INT32TO64(0xA1997B0B,0x4C6A0000);
+  const uint64 ushift=11644473600000000000ULL; // ns between 01-01-1601 and 01-01-1970.
   return itime*(1000000000/TICKS_PER_SECOND)-ushift;
 }
 
@@ -192,8 +191,7 @@ uint64 RarTime::GetUnixNS()
 // Set the high precision Unix time in nanoseconds since 01-01-1970.
 void RarTime::SetUnixNS(uint64 ns)
 {
-  // 11644473600000000000 - number of ns between 01-01-1601 and 01-01-1970.
-  uint64 ushift=INT32TO64(0xA1997B0B,0x4C6A0000);
+  const uint64 ushift=11644473600000000000ULL; // ns between 01-01-1601 and 01-01-1970.
   itime=(ns+ushift)/(1000000000/TICKS_PER_SECOND);
 }
 
@@ -327,14 +325,14 @@ void RarTime::Adjust(int64 ns)
 
 
 #ifndef SFX_MODULE
-const wchar *GetMonthName(int Month)
+const wchar *GetMonthName(uint Month)
 {
   return uiGetMonthName(Month);
 }
 #endif
 
 
-bool IsLeapYear(int Year)
+bool IsLeapYear(uint Year)
 {
   return (Year&3)==0 && (Year%100!=0 || Year%400==0);
 }
