@@ -40,7 +40,7 @@ enum UIMESSAGE_CODE {
   UIERROR_UOWNERGETGROUPID, UIERROR_UOWNERSET, UIERROR_ULINKREAD,
   UIERROR_ULINKEXIST, UIERROR_OPENPRESERVEATIME, UIERROR_READERRTRUNCATED,
   UIERROR_READERRCOUNT, UIERROR_DIRNAMEEXISTS,UIERROR_TRUNCPSW,
-  UIERROR_ADJUSTVALUE,
+  UIERROR_ADJUSTVALUE, UIERROR_SKIPUNSAFELINK,
 
   UIMSG_FIRST,
   UIMSG_STRING, UIMSG_BUILD, UIMSG_RRSEARCH, UIMSG_ANALYZEFILEDATA,
