@@ -677,8 +677,10 @@ int wcsnicomp(const wchar *s1,const wchar *s2,size_t n)
   // If we specify 'n' exceeding the actual string length, CompareString goes
   // beyond the trailing zero and compares garbage. So we need to limit 'n'
   // to real string length.
-  size_t l1=Min(wcslen(s1)+1,n);
-  size_t l2=Min(wcslen(s2)+1,n);
+  size_t sl1=wcslen(s1); // Pre-compute to not call wcslen() in Min() twice.
+  size_t l1=Min(sl1+1,n);
+  size_t sl2=wcslen(s2); // Pre-compute to not call wcslen() in Min() twice.
+  size_t l2=Min(sl2+1,n);
   return CompareStringW(LOCALE_USER_DEFAULT,NORM_IGNORECASE|SORT_STRINGSORT,s1,(int)l1,s2,(int)l2)-2;
 #else
   if (n==0)
